#######################################################################
#
# NCM component for gip daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 0.0.2 : 19/12/04 17:34
#
#  ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::gip;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Path;
use File::Basename;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/gip";

    # Location to install the gip configuration. 
    my $config_file =  $config->getValue("$base/config_file");
    my $config_path = dirname($config_file);

    # Ensure that this directory exists. 
    createAndChownDir("root",$config_path);
    if (! -d $config_path) {
	$self->error("Error creating directory: $config_path");
	return 1;
    }

    # Create a tmp area under this directory with correct user. 
    my $user =  $config->getValue("$base/user");
    my $temp_path =  $config->getValue("$base/temp_path");
    createAndChownDir($user,$temp_path);
    if (! -d $temp_path) {
	$self->error("Error creating directory: $temp_path");
	return 1;
    }

    # FIXME: Descend into this tree to ensure all is owned by the
    # correct user.  Only important if the user changes. 

    # Get the names/paths of the scripts. 
    my $generic_script = $config->getValue("$base/generic_script");
    my $wrapper_script = $config->getValue("$base/wrapper_script");

    # Now create the edg.info script.  Need to split the path and base
    # name for the wrapper script. 
    my $wrapper_path = dirname($wrapper_script);
    my $wrapper_name = basename($wrapper_script,'');

    # Create the contents of the edg.info script.
    my $contents = <<"EOT";
#!/bin/bash
#
# info-globus-ldif.sh
#
#Configures information providers for MDS
#
cat << EOF

dn: Mds-Vo-name=local,o=grid
objectclass: GlobusTop
objectclass: GlobusActiveObject
objectclass: GlobusActiveSearch
type: exec
path: $wrapper_path
base: $wrapper_name
args:
cachetime: 60
timelimit: 20
sizelimit: 50

EOF

EOT
;

    # Now write this out to the edg.info file and ensure it is
    # executable. 
    $config_file = $config->getValue("$base/edginfo_file");
    $self->fillConfigFile($config_file, $contents);
    chmod 0755, $config_file;


    # Get the full list of schemas.
    my $schemas = '';
    if ($config->elementExists("$base/schemas")) {
        my @elements = $config->getElement("$base/schemas")->getList();
	foreach (@elements) {
	    $schemas .= $_->getValue() . "\n";
	}
    }

    # Create the contents of the edg.schemalist script.
    $contents = <<EOT;
#!/bin/bash

cat <<EOF
$schemas
EOF

EOT
;

    # Now write this out to the edg.schemalist file and ensure it is
    # executable. 
    $config_file = $config->getValue("$base/edgschema_file");
    $self->fillConfigFile($config_file, $contents);
    chmod 0755, $config_file;

    # Pull out the location of the configuration file RELATIVE
    # to the path!
    $config_file = $config->getValue("$base/config_file");
    $contents = $self->createContents($config);
    $self->fillConfigFile($config_file, $contents);

    # Pull out the location of the configuration script.
    my $config_script = $config->getValue("$base/config_script");

    # Execute this script passing the location of the config. file.
    # This creates the static LDIF file.
    if (-x $config_script) {
	system("$config_script $config_file");
	if ($?) {
	    $self->error("error running $config_script: " . ($?>>8));
	    return 1;
	}
    } else {
	$self->error("$config_script doesn't exist or isn't executable");
	return 1;
    }

    # Done. 
    return 1; 
}


# Since templating doesn't exist in quattor yet, do this by hand.
sub createContents {

    my ($self,$config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/gip";

    # Save the date.
    my $date = localtime();

    my $contents = "#\n# Generated by ncm-gip on $date\n#\n";

    # The absolute name of the ldif file.
    $contents .= "ldif_file=" . 
	$config->getValue("$base/ldif_file") . "\n";

    # The generic script. 
    $contents .= "generic_script=" . 
	$config->getValue("$base/generic_script") . "\n";

    # The wrapper script. 
    $contents .= "wrapper_script=" . 
	$config->getValue("$base/wrapper_script") . "\n";

    # The tmp directory. 
    $contents .= "temp_path=" . 
	$config->getValue("$base/temp_path") . "\n";

    # List of all of the templates. 
    if ($config->elementExists("$base/templates")) {
        my @elements = $config->getElement("$base/templates")->getList();
	foreach (@elements) {
	    $contents .= "template=" . $_->getValue() . "\n";
	}
    }

    # Add a new line to the output. 
    my $hostname = $config->getValue("$base/hostname");
    $contents .= <<"EOF"

# Common for all
GlueInformationServiceURL: ldap://$hostname:2135/mds-vo-name=local,o=grid

EOF
;

    # Get a list of all of the dynamic scripts.
    my @dscripts;
    if ($config->elementExists("$base/dynamic_scripts")) {
        @dscripts = $config->getElement("$base/dynamic_scripts")->getList();
    }

    # Write out the information for each script.
    foreach my $entry (@dscripts) {
	my %ehash = $entry->getHash();

	# The script and argument. 
	my $script = $ehash{script}->getValue();
	my $args = $ehash{args}->getValue();
	$contents .= "dynamic_script=$script $args\n";

	# Any optional parameters. 
	if (defined($ehash{attributes})) {
	    my %attrs = $ehash{attributes}->getHash();
	    foreach my $key (sort keys %attrs) {
		my @values = $attrs{$key}->getList();
		foreach my $v (@values) {
		    my $value = $v->getValue();
		    $contents .= "$key: $value\n";
		}
	    }
	}
	$contents .= "\n";
    }

    # Get a list of all of the literal entries.
    my @entries;
    if ($config->elementExists("$base/entries")) {
        @entries = $config->getElement("$base/entries")->getList();
    }

    # Write out the information for each entry.
    foreach my $entry (@entries) {
	my %ehash = $entry->getHash();
	my $dn = $ehash{dn}->getValue();
	$contents .= "$dn\n";
	my %attrs = $ehash{attributes}->getHash();
	foreach my $key (sort keys %attrs) {
	    my @values = $attrs{$key}->getList();
	    foreach my $v (@values) {
		my $value = $v->getValue();
		$contents .= "$key: $value\n";
	    }
	}
	$contents .= "\n";
    }

    return $contents;
}


sub fillConfigFile {

    my ($self, $config_file, $contents) = @_;

    # Fill the file with the contents. 
    if ( ! -e $config_file ) {
	
	# Configuration file doesn't exist yet.  Create it. 
	open CONF, ">$config_file";
	print CONF $contents;
	close (CONF);
	$self->log("$config_file created");
	
    } else {
	
	# Already exists. Make backup and create new file. 
	my $result = LC::Check::file( $config_file,
				      backup => ".old",
				      contents => $contents,
				      );
	$self->log("$config_file updated") if $result;
    }
}


# Change ownership by name.
sub createAndChownDir {

    my ($user, $dir) = @_;

    mkpath($dir,0,0755) unless (-e $dir);
    chown((getpwnam($user))[2,3], glob($dir)) if (-d $dir);
}

1;      # Required for PERL modules
